@@ -1,6 +1,5 @@
 package urpm::media;
 
-# $Id: media.pm 270394 2010-07-30 00:48:07Z misc $
 
 use strict;
 use urpm qw(file_from_local_medium is_local_medium);
@@ -552,6 +551,10 @@ sub write_urpmi_cfg {
     };
     remove_passwords_and_write_private_netrc($urpm, $config);
 
+    # urpmi.cfg must be world-readable, else mgaapplet won't be able to read it
+    # as it is executed from the user session. We enforce umask here in the case
+    # where the msec security level is set to 'secure' (which means umask 077).
+    umask 0022;
     urpm::cfg::dump_config($urpm->{config}, $config)
 	or $urpm->{fatal}(6, N("unable to write config file [%s]", $urpm->{config}));
 
@@ -780,6 +783,7 @@ sub is_media_to_add_by_default {
     my ($urpm, $distribconf, $medium, $product_id, $nonfree, $tainted) = @_;
     my $add_by_default = !$distribconf->getvalue($medium, 'noauto');
     my @media_types = split(':', $distribconf->getvalue($medium, 'media_type'));
+    return $add_by_default if !@media_types;
     if ($product_id->{product} eq 'Free') {
 	if (member('non-free', @media_types)) {
 	    $urpm->{log}(N("ignoring non-free medium `%s'", $medium));
@@ -793,7 +797,7 @@ sub is_media_to_add_by_default {
 		$add_by_default = 1;
 		$urpm->{log}(N("un-ignoring non-free medium `%s' b/c nonfree packages are installed", $medium_name));
 	    }
-	    if ($medium_name =~ /Nonfree/ && $tainted) {
+	    if ($medium_name =~ /Tainted/ && $tainted) {
 		$add_by_default = 1;
 		$urpm->{log}(N("un-ignoring tainted medium `%s' b/c tainted packages are installed", $medium_name));
 	    }
@@ -2249,7 +2253,6 @@ sub clean {
 
 1;
 
-__END__
 
 =back
 
@@ -2259,6 +2262,6 @@ Copyright (C) 2005 MandrakeSoft SA
 
 Copyright (C) 2005-2010 Mandriva SA
 
-Copyright (C) 2011-2013 Mageia
+Copyright (C) 2011-2017 Mageia
 
 =cut
